/*
 * Copyright 2004, 2005 Mark Raddatz
 * Copyright 2000, 2001, 2002 Ian Campbell
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include <bmp/plugin.h>
#include <bmp/beepctrl.h>
#include <bmp/util.h>

#include "eggtrayicon.h"
#include "bmp_player.xpm"

struct developer
{
  char *name;
  char *email;
};

struct translator
{
  char *language;
  char *abbr;
  char *name;
  char *email;
};

static struct developer developers[] = {
  {"Mark Raddatz", "mark@xnull.de"},
  {NULL, NULL}
};

static struct translator translators[] = {
  {N_("German"), "de", "Mark Raddatz", "mark@xnull.de"},
  {N_("French"), "fr", "David Le Brun", "david@dyn-ns.net"},
  {N_("Polish"), "pl", "Jacek Wolszczak", "shutdownrunner@o2.pl"},
  {NULL, NULL, NULL, NULL}
};

static void docklet_plugin_init (void);
static void docklet_plugin_about (void);
static void docklet_plugin_cleanup (void);

static GeneralPlugin docklet_plugin = {
  NULL,
  NULL,
  -1,
  "Docklet Plugin",
  docklet_plugin_init,
  docklet_plugin_about,
  NULL,
  docklet_plugin_cleanup
};

EggTrayIcon *docklet = NULL;
GtkWidget *docklet_menu;
GtkTooltips *docklet_tooltips;
gboolean playlist_was_shown;
gboolean eq_was_shown;

GeneralPlugin *
get_gplugin_info (void)
{
  return &docklet_plugin;
}

void
docklet_menu_play_cb (GtkAction * action)
{
  xmms_remote_play (docklet_plugin.xmms_session);
}

void
docklet_menu_pause_cb (GtkAction * action)
{
  xmms_remote_pause (docklet_plugin.xmms_session);
}

void
docklet_menu_stop_cb (GtkAction * action)
{
  xmms_remote_stop (docklet_plugin.xmms_session);
}

void
docklet_menu_previous_cb (GtkAction * action)
{
  xmms_remote_playlist_prev (docklet_plugin.xmms_session);
}

void
docklet_menu_next_cb (GtkAction * action)
{
  xmms_remote_playlist_next (docklet_plugin.xmms_session);
}

void
docklet_menu_preferences_cb (GtkAction * action)
{
  xmms_remote_show_prefs_box (docklet_plugin.xmms_session);
}

void
docklet_menu_quit_cb (GtkAction * action)
{
  xmms_remote_quit (docklet_plugin.xmms_session);
}

GtkActionEntry action_entries[] = {
  {"Play",
   NULL,
   N_("Play"),
   "X",
   N_("Play"),
   G_CALLBACK (docklet_menu_play_cb)},
  {"Pause",
   NULL,
   N_("Pause"),
   "C",
   N_("Pause"),
   G_CALLBACK (docklet_menu_pause_cb)},
  {"Stop",
   GTK_STOCK_STOP,
   N_("Stop"),
   "V",
   N_("Stop"),
   G_CALLBACK (docklet_menu_stop_cb)},
  {"Previous",
   GTK_STOCK_GO_BACK,
   N_("Previous"),
   "Z",
   N_("Previous"),
   G_CALLBACK (docklet_menu_previous_cb)},
  {"Next",
   GTK_STOCK_GO_FORWARD,
   N_("Next"),
   "B",
   N_("Next"),
   G_CALLBACK (docklet_menu_next_cb)},
  {"Preferences",
   GTK_STOCK_PREFERENCES,
   N_("Preferences"),
   "<control>P",
   N_("Preferences"),
   G_CALLBACK (docklet_menu_preferences_cb)},
  {"Quit",
   GTK_STOCK_QUIT,
   N_("Quit"),
   "<control>Q",
   N_("Quit"),
   G_CALLBACK (docklet_menu_quit_cb)}
};

gchar *ui_info =
  "<ui>"
  "  <popup name=\"Docklet\">"
  "    <menuitem action=\"Play\"/>"
  "    <menuitem action=\"Pause\"/>"
  "    <menuitem action=\"Stop\"/>"
  "    <menuitem action=\"Previous\"/>"
  "    <menuitem action=\"Next\"/>"
  "    <separator/>"
  "    <menuitem action=\"Preferences\"/>"
  "    <separator/>"
  "    <menuitem action=\"Quit\"/>"
  "  </popup>"
  "</ui>";


GtkWidget *
docklet_menu_new (void)
{
  GtkUIManager *ui_manager;
  GtkActionGroup *action_group;

  ui_manager = gtk_ui_manager_new ();

  action_group = gtk_action_group_new ("DockletActions");
#ifdef ENABLE_NLS
  gtk_action_group_set_translation_domain (action_group, PACKAGE);
#endif
  gtk_action_group_add_actions (action_group, action_entries,
                                G_N_ELEMENTS (action_entries), NULL);

  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
  gtk_ui_manager_add_ui_from_string (ui_manager, ui_info, -1, NULL);

  return gtk_ui_manager_get_widget (ui_manager, "/Docklet");
}

void
toggle_all_windows (void)
{
  gboolean window_shown;
  window_shown = xmms_remote_is_main_win (docklet_plugin.xmms_session);

  if (window_shown)
    {
      // hide windows
      playlist_was_shown =
        xmms_remote_is_pl_win (docklet_plugin.xmms_session);
      eq_was_shown = xmms_remote_is_eq_win (docklet_plugin.xmms_session);

      if (playlist_was_shown)
        xmms_remote_pl_win_toggle (docklet_plugin.xmms_session,
                                   !playlist_was_shown);

      if (eq_was_shown)
        xmms_remote_eq_win_toggle (docklet_plugin.xmms_session,
                                   !eq_was_shown);

      xmms_remote_main_win_toggle (docklet_plugin.xmms_session,
                                   !window_shown);
    }
  else
    {
      // restore windows
      xmms_remote_main_win_toggle (docklet_plugin.xmms_session,
                                   !window_shown);

      if (eq_was_shown)
        xmms_remote_eq_win_toggle (docklet_plugin.xmms_session, eq_was_shown);

      if (playlist_was_shown)
        xmms_remote_pl_win_toggle (docklet_plugin.xmms_session,
                                   playlist_was_shown);
    }
}

gboolean
docklet_tooltips_cb (gpointer data)
{
  gint pos;
  gchar *title, *noutf8_title;
  static gchar *last_title = NULL;

  if (docklet == NULL)
    return FALSE;

  if (last_title == NULL)
    last_title = g_utf8_casefold (g_strdup (""), -1);

  pos = xmms_remote_get_playlist_pos (docklet_plugin.xmms_session);
  noutf8_title =
    xmms_remote_get_playlist_title (docklet_plugin.xmms_session, pos);

  if (noutf8_title == NULL)
    return TRUE;

  title = g_utf8_casefold (noutf8_title, -1);
  g_free (noutf8_title);

  /* change tooltip only if title really changed */

  if (g_utf8_collate (last_title, title) != 0)
    {
      gtk_tooltips_set_tip (GTK_TOOLTIPS (docklet_tooltips),
                            GTK_WIDGET (docklet), title, NULL);

      g_free (last_title);
      last_title = title;
    }
  else
    g_free (title);

  return TRUE;
}

void
docklet_clicked_cb (GtkWidget * button, GdkEventButton * event, void *data)
{
  switch (event->button)
    {
    case 1:
      toggle_all_windows ();
      break;
    case 2:
      break;
    case 3:
      gtk_menu_popup (GTK_MENU (docklet_menu), NULL, NULL, NULL, NULL,
                      event->button, event->time);
      break;
    }
}

static void
docklet_plugin_init (void)
{
  GdkPixbuf *icon, *icon_small;
  GtkWidget *image, *box;

#ifdef ENABLE_NLS
  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALE_DIR);
  bind_textdomain_codeset (PACKAGE, "UTF-8");
  textdomain (PACKAGE);
#endif

  docklet = egg_tray_icon_new ("bmp");

  /* create icon image */
  icon = gdk_pixbuf_new_from_xpm_data ((const gchar **) bmp_player_icon);
  icon_small = gdk_pixbuf_scale_simple (icon, 22, 22, GDK_INTERP_BILINEAR);
  image = gtk_image_new_from_pixbuf (icon_small);

  g_object_unref (G_OBJECT (icon));
  g_object_unref (G_OBJECT (icon_small));

  box = gtk_event_box_new ();
  g_signal_connect (G_OBJECT (box), "button-press-event",
                    G_CALLBACK (docklet_clicked_cb), NULL);

  gtk_container_add (GTK_CONTAINER (box), image);
  gtk_container_add (GTK_CONTAINER (docklet), box);

  docklet_menu = (GtkWidget *) docklet_menu_new ();

  docklet_tooltips = gtk_tooltips_new ();
  g_timeout_add (1000, docklet_tooltips_cb, NULL);

  gtk_widget_show_all (GTK_WIDGET (docklet));
}

static void
docklet_plugin_about (void)
{
  static GtkWidget *about_dialog;
  GString *str;
  int i;

  if (about_dialog != NULL)
    return;

#ifdef ENABLE_NLS
  setlocale (LC_ALL, "");
  bindtextdomain (PACKAGE, LOCALE_DIR);
  bind_textdomain_codeset (PACKAGE, "UTF-8");
  textdomain (PACKAGE);
#endif

  str = g_string_sized_new (384);
  g_string_printf (str, _("Beep Media Player Docklet Plugin %s\n\n"),
                   VERSION);
  g_string_append_printf (str, _("Displays a system tray icon (in GNOME or "
                                 "KDE for example) to allow fast access to "
                                 "commonly used functions, and to toggle "
                                 "display of the Beep Media Player "
                                 "window.\n\n"));
  g_string_append_printf (str, "URL: %s\n\n",
                          "http://mark.xnull.de/bmp-docklet.php");

  g_string_append_printf (str, "%s:\n", _("Developers"));
  for (i = 0; developers[i].name != NULL; i++)
    {
      g_string_append_printf (str, "%s", developers[i].name);
      if (developers[i].email != NULL)
        g_string_append_printf (str, " <%s>", developers[i].email);
      g_string_append_printf (str, "\n");
    }
  g_string_append_printf (str, "\n");

  g_string_append_printf (str, "%s:\n", _("Translators"));
  for (i = 0; translators[i].language != NULL; i++)
    {
      g_string_append_printf (str, "%s (%s) - %s", _(translators[i].language),
                              translators[i].abbr, translators[i].name);
      if (translators[i].email != NULL)
        g_string_append_printf (str, " <%s>", translators[i].email);
      g_string_append_printf (str, "\n");
    }
  g_string_append_printf (str, "\n");

  about_dialog =
    bmp_info_dialog (_("About Beep Media Player Docklet"), str->str, "Close",
                     FALSE, NULL, NULL);
  g_string_free (str, TRUE);

  g_signal_connect (G_OBJECT (about_dialog), "destroy",
                    G_CALLBACK (gtk_widget_destroyed), &about_dialog);
}

static void
docklet_plugin_cleanup (void)
{
  if (docklet_tooltips != NULL)
    {
      gtk_object_destroy (GTK_OBJECT (docklet_tooltips));
      docklet_tooltips = NULL;
    }

  if (docklet_menu != NULL)
    {
      gtk_widget_destroy (GTK_WIDGET (docklet_menu));
      docklet_menu = NULL;
    }

  if (docklet != NULL)
    {
      gtk_widget_destroy (GTK_WIDGET (docklet));
      docklet = NULL;
    }
}
